/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier;

/**
 * The NativeVerifier class implements a main(String[] args) method that's
 * roughly compatible to the one in the Verifier class, but that uses the
 * JVM's internal verifier for its class file verification.
 * This can be used for comparison runs between the JVM-internal verifier
 * and JustIce.
 *
 * @version $Id: NativeVerifier.java,v 1.13 2001/08/30 08:45:41 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
public abstract class NativeVerifier{

	/**
	 * This class must not be instantiated.
	 */
	private NativeVerifier(){
	}
	
	/**
	 * Works only on the first argument.
	 */
	public static void main(String [] args){
		if (args.length != 1){
			System.out.println("Verifier front-end: need exactly one argument.");
			System.exit(1);
		}

		int dotclasspos = args[0].lastIndexOf(".class");
		if (dotclasspos != -1) args[0] = args[0].substring(0,dotclasspos);
		args[0] = args[0].replace('/','.');
		//System.out.println(args[0]);

		
		try{
			Class.forName(args[0]);
		}
		catch(ExceptionInInitializerError eiie){ //subclass of LinkageError!
			System.out.println("NativeVerifier: ExceptionInInitializerError encountered on '"+args[0]+"'.");
			System.out.println(eiie);
			System.exit(1);		
		}
		catch(LinkageError le){
			System.out.println("NativeVerifier: LinkageError encountered on '"+args[0]+"'.");
			System.out.println(le);
			System.exit(1);
		}
		catch(ClassNotFoundException cnfe){
			System.out.println("NativeVerifier: FILE NOT FOUND: '"+args[0]+"'.");
			System.exit(1);
		}
		catch(Throwable t){
			System.out.println("NativeVerifier: Unspecified verification error on'"+args[0]+"'.");
			System.exit(1);
		}
		
		System.out.println("NativeVerifier: Class file '"+args[0]+"' seems to be okay.");
		System.exit(0);

	}
}
