/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier;

import de.fub.bytecode.verifier.*;
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.*;

/**
 * This class has a main method implementing a demonstration program
 * of how to use the VerifierFactoryObserver. It transitively verifies
 * all class files encountered; this may take up a lot of time and,
 * more notably, memory.
 *
 * @version $Id: TransitiveHull.java,v 1.6 2001/09/12 14:57:39 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
public class TransitiveHull implements VerifierFactoryObserver{

	/** Used for indentation. */
	private static int indent = 0;

	/** Not publicly instantiable. */
	private TransitiveHull(){
	}
	
	/* Implementing VerifierFactoryObserver. */
	public void notify(String classname){

		System.gc(); // avoid swapping if possible.

		for (int i=0; i<indent; i++){
			System.out.print(" ");
		}
		System.out.println(classname);
		indent += 1;

		Verifier v = VerifierFactory.getVerifier(classname);
	
		VerificationResult vr;
		vr = v.doPass1();
		if (vr != VerificationResult.VR_OK) //System.exit(1);
			System.out.println("Pass 1:\n"+vr);

		vr = v.doPass2();
      if (vr != VerificationResult.VR_OK) //System.exit(1);
			System.out.println("Pass 2:\n"+vr);

		if (vr == VerificationResult.VR_OK){
			JavaClass jc = Repository.lookupClass(v.getClassName());
			for (int i=0; i<jc.getMethods().length; i++){
				vr = v.doPass3a(i);
				if (vr != VerificationResult.VR_OK) //System.exit(1);
					System.out.println(v.getClassName()+", Pass 3a, method "+i+" ['"+jc.getMethods()[i]+"']:\n"+vr);

				vr = v.doPass3b(i);
				if (vr != VerificationResult.VR_OK) //System.exit(1);
					System.out.println(v.getClassName()+", Pass 3b, method number "+i+" ['"+jc.getMethods()[i]+"']:\n"+vr);
			}
		}

		indent -= 1;
	}

	/**
	 * This method implements a demonstration program
	 * of how to use the VerifierFactoryObserver. It transitively verifies
	 * all class files encountered; this may take up a lot of time and,
	 * more notably, memory.
	 */
	public static void main(String[] args){
		if (args.length != 1){
			System.out.println("Need exactly one argument: The root class to verify.");
			System.exit(1);
		}

		int dotclasspos = args[0].lastIndexOf(".class");
		if (dotclasspos != -1) args[0] = args[0].substring(0,dotclasspos);
		args[0] = args[0].replace('/', '.');
	
		TransitiveHull th = new TransitiveHull();
		VerifierFactory.addObserver(th);
		VerifierFactory.getVerifier(args[0]); // the observer is called back and does the actual trick.
		VerifierFactory.removeObserver(th);
	}
}
