/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier.exc;

/**
 * Instances of this class should never be thrown. When such an instance is thrown,
 * this is due to an INTERNAL ERROR of BCEL's class file verifier &quot;JustIce&quot;.
 *
 * @version $Id: AssertionViolatedException.java,v 1.14 2001/08/30 08:45:42 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
public final class AssertionViolatedException extends RuntimeException{
	/** The error message. */
	private String detailMessage;
	/** Constructs a new AssertionViolatedException with null as its error message string. */
	public AssertionViolatedException(){
		super();
	}
	/**
	 * Constructs a new AssertionViolatedException with the specified error message preceded
	 * by &quot;INTERNAL ERROR: &quot;.
	 */
	public AssertionViolatedException(String message){
		super(message = "INTERNAL ERROR: "+message); // Thanks to Java, the constructor call here must be first.
		detailMessage=message;
	}
	/** Extends the error message with a string before ("pre") and after ("post") the
	    'old' error message. All of these three strings are allowed to be null, and null
	    is always replaced by the empty string (""). In particular, after invoking this
	    method, the error message of this object can no longer be null.
	*/
	public void extendMessage(String pre, String post){
		if (pre  == null) pre="";
		if (detailMessage == null) detailMessage="";
		if (post == null) post="";
		detailMessage = pre+detailMessage+post;
	}
	/**
	 * Returns the error message string of this AssertionViolatedException object.
	 * @return the error message string of this AssertionViolatedException.
	 */
	public String getMessage(){
		return detailMessage;
	}

	/** 
	 * DO NOT USE. It's for experimental testing during development only.
	 */
	public static void main(String[] args){
		AssertionViolatedException ave = new AssertionViolatedException("Oops!");
		ave.extendMessage("\nFOUND:\n\t","\nExiting!!\n");
		throw ave;
	}

	/**
	 * Returns the backtrace of this AssertionViolatedException as a String.
	 * @return The backtrace of this AssertionViolatedException as a String.
	 */
	public String getStackTrace(){
		return Utility.getStackTrace(this);
	}

}
