/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier.statics;

import de.fub.bytecode.Constants;
import de.fub.bytecode.generic.Type;
import de.fub.bytecode.verifier.exc.*;

/**
 * A utility class holding the information about
 * the names and the types of the local variables in
 * a given method.
 *
 * @version $Id: LocalVariablesInfo.java,v 1.7 2001/08/30 08:45:42 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
public class LocalVariablesInfo{
	
	/** The information about the local variables is stored here. */
	private LocalVariableInfo[] localVariableInfos;

	/**
	 * The ints in the list represent code offsets where either instructions must start
	 * or the offset is the length of the code array. This will be verified in Pass 3a.
	 */
	private IntList instruction_offsets = new IntList();
	
	/** The constructor. */
	LocalVariablesInfo(int max_locals){
		localVariableInfos = new LocalVariableInfo[max_locals];
		for (int i=0; i<max_locals; i++){
			localVariableInfos[i] = new LocalVariableInfo();
		}
	}

	/** Returns the LocalVariableInfo for the given slot. */
	public LocalVariableInfo getLocalVariableInfo(int slot){
		if (slot < 0 || slot >= localVariableInfos.length){
			throw new AssertionViolatedException("Slot number for local variable information out of range.");
		}
		return localVariableInfos[slot];
	}

	/**
	 * Adds information about the local variable in slot 'slot'. Automatically 
	 * adds information for slot+1 if 't' is Type.LONG or Type.DOUBLE.
	 * @throws LocalVariableInfoInconsistentException if the new information conflicts
	 *         with already gathered information.
	 */
	public void add(int slot, String name, int startpc, int length, Type t) throws LocalVariableInfoInconsistentException{
		// The add operation on LocalVariableInfo may throw the '...Inconsistent...' exception, we don't throw it explicitely here.
		
		if (slot < 0 || slot >= localVariableInfos.length){
			throw new AssertionViolatedException("Slot number for local variable information out of range.");
		}

		localVariableInfos[slot].add(name, startpc, length, t);
		if (t == Type.LONG) localVariableInfos[slot+1].add(name, startpc, length, LONG_Upper.theInstance());
		if (t == Type.DOUBLE) localVariableInfos[slot+1].add(name, startpc, length, DOUBLE_Upper.theInstance());
	}
}
