/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier.structurals;

import de.fub.bytecode.generic.*;
import de.fub.bytecode.verifier.exc.*;
import java.util.ArrayList;

/**
 * This class represents a JVM execution frame; that means,
 * a local variable array and an operand stack.
 *
 * @version $Id: Frame.java,v 1.8 2001/09/03 08:15:12 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
 
public class Frame{

	/**
	 * For instance initialization methods, it is important to remember
	 * which instance it is that is not initialized yet. It will be
	 * initialized invoking another constructor later.
	 * NULL means the instance already *is* initialized.
	 */
	protected static UninitializedObjectType _this;

	/**
	 *
	 */
	private LocalVariables locals;

	/**
	 *
	 */
	private OperandStack stack;

	/**
	 *
	 */
	public Frame(int maxLocals, int maxStack){
		locals = new LocalVariables(maxLocals);
		stack = new OperandStack(maxStack);
	}

	/**
	 *
	 */
	public Frame(LocalVariables locals, OperandStack stack){
		this.locals = locals;
		this.stack = stack;
	}

	/**
	 *
	 */
	protected Object clone(){
		Frame f = new Frame(locals.getClone(), stack.getClone());
		return f;
	}

	/**
	 *
	 */
	public Frame getClone(){
		return (Frame) clone();
	}

	/**
	 *
	 */
	public LocalVariables getLocals(){
		return locals;
	}

	/**
	 *
	 */
	public OperandStack getStack(){
		return stack;
	}

	/**
	 *
	 */
	public boolean equals(Object o){
		if (!(o instanceof Frame)) return false; // implies "null" is non-equal.
		Frame f = (Frame) o;
		return this.stack.equals(f.stack) && this.locals.equals(f.locals);
	}

	/**
	 * Returns a String representation of the Frame instance.
	 */
	public String toString(){
		String s="Local Variables:\n";
		s += locals;
		s += "OperandStack:\n";
		s += stack;
		return s;
	}
}
